// Standard includes (may not be required):
#include "stdio.h"

// lwip include (required):
#include "tcp.h"

// Response to server request
static char packetData[] = "";

static void popi_start(void *arg, struct tcp_pcb *pcb, struct pbuf *p) {
  
  char *data;
  
  // If we got a NULL pbuf in p, the remote end has closed the connection.
  if(p != NULL) {
    
    // The payload pointer in the pbuf contains the data in the TCP segment.
    data = p->payload;
    


    // packetData is later written to the stack
    packetData = tcpClientProcess(data);
      
      //if enter was not received (newline escape char)
      if(strlen(packetData) != 0) {
   
		//send packetData response to pop server
		tcp_write(pcb, packetData, sizeof(packetData), 0);
  		
  	  } else {
	  	  
	  	  // Close the connection on our end.
		  tcp_close(pcb);
	  	  	  
  	  }

    // Free the pbuf.
    pbuf_free(p);
  
    } else {
  
  	// Connection closed on other end, so close ours also.
  	tcp_close(pcb);
   
  	}

}

// This is the callback function that is called when a connection has been accepted.
static void popi_accept(void *arg, struct tcp_pcb *pcb) {
  
  // Set up the function popi_start so that it is called when a packet is received on the tcp connection
  tcp_recv(pcb, popi_start, NULL);
  
}


// This is the callback function that is called when a connection has been accepted.
static void popi_connect(void *arg, struct tcp_pcb *pcb) {
  
  // Set up the function popi_start to be called when a packet is received on the tcp connection
  // after the handshake this method is called to handle data passed from server to client
  // Handshaking is handle internally by lwip
  tcp_recv(pcb, popi_start, NULL);
  
}
	
// The initialization function.
void popi_init(void) {
  
  struct tcp_pcb *pcb;
  struct ip_addr ipaddr;
  u16_t port = 110;
  
  //Create a new TCP PCB.
  pcb = tcp_pcb_new();
  
  //Bind the PCB to TCP port 110
  //TODO: Change this to operating port of pop3 server
  tcp_bind(pcb, NULL, 110);

  //Configure the ip of the outgoing message
  IP4_ADDR(&ipaddr, 160,39,190,152);
  
  //Set up popi_connect function to be called when a new connection arrives
  tcp_connect(pcb, NULL, 110, popi_connect);
  
}
