--------------------------------------------------------------------------------
-- lmb_instruction_controller_wrapper.vhd
--------------------------------------------------------------------------------
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;

-- synthesis translate_off
LIBRARY UNISIM;
USE UNISIM.VCOMPONENTS.ALL;
-- synthesis translate_on

LIBRARY lmb_bram_if_cntlr_v1_00_b;
USE lmb_bram_if_cntlr_v1_00_b.ALL;

ENTITY lmb_instruction_controller_wrapper IS
PORT (
	-- instance lmb_instruction_controller
	LMB_Clk : IN STD_LOGIC;
	LMB_Rst : IN STD_LOGIC;
	LMB_ABus : IN STD_LOGIC_VECTOR(0 TO 31);
	LMB_WriteDBus : IN STD_LOGIC_VECTOR(0 TO 31);
	LMB_AddrStrobe : IN STD_LOGIC;
	LMB_ReadStrobe : IN STD_LOGIC;
	LMB_WriteStrobe : IN STD_LOGIC;
	LMB_BE : IN STD_LOGIC_VECTOR(0 TO 3);
	Sl_DBus : OUT STD_LOGIC_VECTOR(0 TO 31);
	Sl_Ready : OUT STD_LOGIC;
	BRAM_Rst_A : OUT STD_LOGIC;
	BRAM_Clk_A : OUT STD_LOGIC;
	BRAM_EN_A : OUT STD_LOGIC;
	BRAM_WEN_A : OUT STD_LOGIC_VECTOR(0 TO 3);
	BRAM_Addr_A : OUT STD_LOGIC_VECTOR(0 TO 31);
	BRAM_Din_A : IN STD_LOGIC_VECTOR(0 TO 31);
	BRAM_Dout_A : OUT STD_LOGIC_VECTOR(0 TO 31));
END lmb_instruction_controller_wrapper;

ARCHITECTURE IMP OF lmb_instruction_controller_wrapper IS

--------------------------------------------------------------------------------
COMPONENT lmb_bram_if_cntlr IS
GENERIC (
	C_BASEADDR : STD_LOGIC_VECTOR(0 TO 31) := X"00000000";
	C_HIGHADDR : STD_LOGIC_VECTOR(0 TO 31) := X"00001fff";
	C_MASK : STD_LOGIC_VECTOR(0 TO 31) := X"00800000";
	C_LMB_AWIDTH : INTEGER := 32;
	C_LMB_DWIDTH : INTEGER := 32);
PORT (
	LMB_Clk : IN STD_LOGIC;
	LMB_Rst : IN STD_LOGIC;
	LMB_ABus : IN STD_LOGIC_VECTOR(0 TO C_LMB_AWIDTH-1);
	LMB_WriteDBus : IN STD_LOGIC_VECTOR(0 TO C_LMB_DWIDTH-1);
	LMB_AddrStrobe : IN STD_LOGIC;
	LMB_ReadStrobe : IN STD_LOGIC;
	LMB_WriteStrobe : IN STD_LOGIC;
	LMB_BE : IN STD_LOGIC_VECTOR(0 TO C_LMB_DWIDTH/8-1);
	Sl_DBus : OUT STD_LOGIC_VECTOR(0 TO C_LMB_DWIDTH-1);
	Sl_Ready : OUT STD_LOGIC;
	BRAM_Rst_A : OUT STD_LOGIC;
	BRAM_Clk_A : OUT STD_LOGIC;
	BRAM_EN_A : OUT STD_LOGIC;
	BRAM_WEN_A : OUT STD_LOGIC_VECTOR(0 TO (C_LMB_DWIDTH/8)-1);
	BRAM_Addr_A : OUT STD_LOGIC_VECTOR(0 TO C_LMB_AWIDTH-1);
	BRAM_Din_A : IN STD_LOGIC_VECTOR(0 TO C_LMB_DWIDTH-1);
	BRAM_Dout_A : OUT STD_LOGIC_VECTOR(0 TO C_LMB_DWIDTH-1));
END COMPONENT;

--------------------------------------------------------------------------------
-- internal signals
--------------------------------------------------------------------------------

BEGIN

--------------------------------------------------------------------------------
-- Power assignments
--------------------------------------------------------------------------------
-- Constant assignments
--------------------------------------------------------------------------------
-- Top-level port assignments
--------------------------------------------------------------------------------
-- Lower-level assignments

--------------------------------------------------------------------------------
lmb_bram_if_cntlr_0_i : lmb_bram_if_cntlr
GENERIC MAP (
	C_BASEADDR => X"00000000",
	C_HIGHADDR => X"00001fff",
	C_MASK => X"00800000",
	C_LMB_AWIDTH => 32,
	C_LMB_DWIDTH => 32)
PORT MAP (
	LMB_Clk => LMB_Clk,
	LMB_Rst => LMB_Rst,
	LMB_ABus => LMB_ABus,
	LMB_WriteDBus => LMB_WriteDBus,
	LMB_AddrStrobe => LMB_AddrStrobe,
	LMB_ReadStrobe => LMB_ReadStrobe,
	LMB_WriteStrobe => LMB_WriteStrobe,
	LMB_BE => LMB_BE,
	Sl_DBus => Sl_DBus,
	Sl_Ready => Sl_Ready,
	BRAM_Rst_A => BRAM_Rst_A,
	BRAM_Clk_A => BRAM_Clk_A,
	BRAM_EN_A => BRAM_EN_A,
	BRAM_WEN_A => BRAM_WEN_A,
	BRAM_Addr_A => BRAM_Addr_A,
	BRAM_Din_A => BRAM_Din_A,
	BRAM_Dout_A => BRAM_Dout_A);

END ARCHITECTURE IMP;
