--------------------------------------------------------------------------------
-- myuart_wrapper.vhd
--------------------------------------------------------------------------------
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;

-- synthesis translate_off
LIBRARY UNISIM;
USE UNISIM.VCOMPONENTS.ALL;
-- synthesis translate_on

LIBRARY opb_uartlite_v1_00_b;
USE opb_uartlite_v1_00_b.ALL;

ENTITY myuart_wrapper IS
PORT (
	-- instance myuart
	Interrupt : OUT STD_LOGIC;
	OPB_Clk : IN STD_LOGIC;
	RX : IN STD_LOGIC;
	TX : OUT STD_LOGIC;
	OPB_Rst : IN STD_LOGIC;
	OPB_ABus : IN STD_LOGIC_VECTOR(0 TO 31);
	OPB_BE : IN STD_LOGIC_VECTOR(0 TO 3);
	OPB_RNW : IN STD_LOGIC;
	OPB_select : IN STD_LOGIC;
	OPB_seqAddr : IN STD_LOGIC;
	OPB_DBus : IN STD_LOGIC_VECTOR(0 TO 31);
	UART_DBus : OUT STD_LOGIC_VECTOR(0 TO 31);
	UART_errAck : OUT STD_LOGIC;
	UART_retry : OUT STD_LOGIC;
	UART_toutSup : OUT STD_LOGIC;
	UART_xferAck : OUT STD_LOGIC);
END myuart_wrapper;

ARCHITECTURE IMP OF myuart_wrapper IS

--------------------------------------------------------------------------------
COMPONENT opb_uartlite IS
GENERIC (
	C_BASEADDR : STD_LOGIC_VECTOR := X"feff0100";
	C_HIGHADDR : STD_LOGIC_VECTOR := X"feff01ff";
	C_OPB_DWIDTH : INTEGER := 32;
	C_OPB_AWIDTH : INTEGER := 32;
	C_DATA_BITS : INTEGER RANGE 5 TO 8 := 8;
	C_CLK_FREQ : INTEGER := 50_000_000;
	C_BAUDRATE : INTEGER := 9600;
	C_USE_PARITY : INTEGER := 0;
	C_ODD_PARITY : INTEGER := 1);
PORT (
	Interrupt : OUT STD_LOGIC;
	OPB_Clk : IN STD_LOGIC;
	RX : IN STD_LOGIC;
	TX : OUT STD_LOGIC;
	OPB_Rst : IN STD_LOGIC;
	OPB_ABus : IN STD_LOGIC_VECTOR(0 TO C_OPB_AWIDTH-1);
	OPB_BE : IN STD_LOGIC_VECTOR(0 TO C_OPB_DWIDTH/8-1);
	OPB_RNW : IN STD_LOGIC;
	OPB_select : IN STD_LOGIC;
	OPB_seqAddr : IN STD_LOGIC;
	OPB_DBus : IN STD_LOGIC_VECTOR(0 TO C_OPB_DWIDTH-1);
	UART_DBus : OUT STD_LOGIC_VECTOR(0 TO C_OPB_DWIDTH-1);
	UART_errAck : OUT STD_LOGIC;
	UART_retry : OUT STD_LOGIC;
	UART_toutSup : OUT STD_LOGIC;
	UART_xferAck : OUT STD_LOGIC);
END COMPONENT;

--------------------------------------------------------------------------------
-- internal signals
--------------------------------------------------------------------------------

BEGIN

--------------------------------------------------------------------------------
-- Power assignments
--------------------------------------------------------------------------------
-- Constant assignments
--------------------------------------------------------------------------------
-- Top-level port assignments
--------------------------------------------------------------------------------
-- Lower-level assignments

--------------------------------------------------------------------------------
opb_uartlite_0_i : opb_uartlite
GENERIC MAP (
	C_BASEADDR => X"feff0100",
	C_HIGHADDR => X"feff01ff",
	C_OPB_DWIDTH => 32,
	C_OPB_AWIDTH => 32,
	C_DATA_BITS => 8,
	C_CLK_FREQ => 50_000_000,
	C_BAUDRATE => 9600,
	C_USE_PARITY => 0,
	C_ODD_PARITY => 1)
PORT MAP (
	Interrupt => Interrupt,
	OPB_Clk => OPB_Clk,
	RX => RX,
	TX => TX,
	OPB_Rst => OPB_Rst,
	OPB_ABus => OPB_ABus,
	OPB_BE => OPB_BE,
	OPB_RNW => OPB_RNW,
	OPB_select => OPB_select,
	OPB_seqAddr => OPB_seqAddr,
	OPB_DBus => OPB_DBus,
	UART_DBus => UART_DBus,
	UART_errAck => UART_errAck,
	UART_retry => UART_retry,
	UART_toutSup => UART_toutSup,
	UART_xferAck => UART_xferAck);

END ARCHITECTURE IMP;
