------------------------------------------------------------------------------
--
-- Simple OPB peripheral: a ETHENET controller
--
-- YASHKET GUPTA JAVIER COCA, ADAM
-- yg2113@cs.columbia.edu
--
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;

library UNISIM;
use UNISIM.Vcomponents.all;


entity sram_ethernet is

  
  generic (
    C_OPB_AWIDTH : integer                   := 32;
    C_OPB_DWIDTH : integer                   := 32;
    C_BASEADDR   : std_logic_vector(0 to 31) := X"00000000";
    C_HIGHADDR   : std_logic_vector(0 to 31) := X"FFFFFFFF"
           );

  port (

    --ports between peripheral and OPB
    OPB_Clk     : in  std_logic;
    OPB_Rst     : in  std_logic;
    OPB_ABus    : in  std_logic_vector(0 to C_OPB_AWIDTH-1);
    OPB_BE      : in  std_logic_vector(0 to C_OPB_DWIDTH/8-1);  -- not using this
    OPB_DBus    : in  std_logic_vector(0 to C_OPB_DWIDTH-1);
    OPB_RNW     : in  std_logic;
    OPB_select  : in  std_logic;
    OPB_seqAddr : in  std_logic;        -- Sequential Address
    Sln_DBus    : out std_logic_vector(0 to C_OPB_DWIDTH-1);
    Sln_errAck  : out std_logic;        -- (unused)
    Sln_retry   : out std_logic;        -- (unused)
    Sln_toutSup : out std_logic;        -- Timeout suppress
    Sln_xferAck : out std_logic;       -- Transfer acknowledge

       --ports between peripheral and Ethernet
    ETH_CS       : out std_logic;
    ETH_IOCS     : in  std_logic;
    ETH_RDY_DTAK : in  std_logic;
    ETH_IREQ     : in  std_logic;
    
 --    ETH_data     : inout std_logic_vector(15 downto 0)

     --ports between peripheral and SRAM
     -- IOR IOW AEN_PSEN BHE address and data
    RAM_CE          : out std_logic;
    PB_OE_IOR          : out std_logic;
    PB_WE_IOW          : out std_logic;
    PB_UB_BHE          : out std_logic;
    PB_LB_AEN          : out std_logic;
    PB_D   : inout std_logic_vector(0 to 15);
    PB_A   : out std_logic_vector(0 to 17);

    FLASH_CE : out std_logic;
    SDRAM_CE :out std_logic;
    ADC_OE :out std_logic;
    AU_CSN :out std_logic;
    USB_CS :out std_logic;
    NV_CS0 :out std_logic;
    NV_CS1 :out std_logic
    
    );
end sram_ethernet;

architecture Behavioral of sram_ethernet is

  
--deleted old signals from bram

  type opb_state is( Idle,
  Select_Ram,
  Select_Ethernet,
  read_ram_16_a,
  read_ram_16_b,  
  read_ram_32_a,  
  read_ram_32_b,  
  read_ram_32_c,  
  write_ram_32 ,  
  write_ETH_a,
 write_ETH_b,
 write_ETH_c,
 write_ETH_d,
--  write_ETH_e,                                                
  Read_ETH_a,     
  Read_ETH_b,     
  Read_ETH_c,
  Read_ETH_d,
--  Read_ETH_e,                                        
  Xfer);

  signal present_state, next_state : opb_state;

-- new signals

  signal CS, ETHcs, RAMcs, rnw, ub, lb, FSM_tristate, ub_tmp, lb_tmp  : std_logic;
  signal FSM_CS_RAM, FSM_CS_ETH, fsm_oe_ior, fsm_we_iow, sel32, sl_ld_hi, sl_ld_lo : std_logic;
  signal pb_ub_i, pb_lb_i, ram_cs_i, eth_cs_i, pb_oe_i, pb_we_i : std_logic;
  signal pb_d_i , data_fix, data_out , pb_d_o,  data_low, tristate_i, data_high  : std_logic_vector(0 to 15);
  signal addr, pb_a_i, addr_s32 : std_logic_vector(0 to 17);
  signal be : std_logic_vector(0 to 3);

  component OBUF_F_24
    port (
      O : out std_ulogic;
      I : in std_ulogic);
  end component;

  component IOBUF_F_24
    port (
      O : out std_ulogic;
      IO : inout std_ulogic;
      I : in std_ulogic;
      T : in std_ulogic);
  end component;

  component FDCE
   port (C : in std_logic;
                CLR : in std_logic;
                CE : in std_logic;
                D : in std_logic;
                Q : out std_logic);
end component;

 component FDPE
   port (C : in std_logic;
                PRE : in std_logic;
                CE : in std_logic;
                D : in std_logic;
                Q : out std_logic);
end component;
  
attribute iob : string;
attribute iob of FDCE : component is "true";
attribute iob of FDPE : component is "true";
  
begin

 FLASH_CE <= '1';
 SDRAM_CE <= '1';
 ADC_OE <= '1';
 AU_CSN <= '1';
 USB_CS <= '1';
 NV_CS0 <= '1';
 NV_CS1 <= '1';
  
 CS <= OPB_Select  when OPB_ABus(0 to 11) = C_BASEADDR(0 to 11) else
       '0';

 ETHcs <= CS and OPB_ABus(12);
 RAMcs <= CS and (not OPB_ABus(12));

 process (OPB_Rst, OPB_Clk)             -- address
 begin  -- process
   if OPB_Clk'event and OPB_Clk='1' then
     addr <= OPB_Abus( 13 to 30);
     be <= OPB_BE(0 to 3);
     rnw <= OPB_rnw;
   end if;
   if OPB_Rst='1' then
     addr <= (others => '0');
     be <= (others => '0');
     rnw <= '0';
   end if;
 end process;


  addr_s32 <= addr(0 to 16) & (addr(17) or sel32);
   
  lb_tmp <= NOT (be(3) or be(1));
  ub_tmp <= NOT (be(2) or be(0));

-- PAD IO's 
   
-- process (OPB_Rst, OPB_Clk)
-- begin  -- process
--   if OPB_Clk'event and OPB_Clk='1' then
--    pb_a_i <= addr_s32;
--    pb_ub_i <= ub;
--    pb_lb_i <= lb;
--    ram_cs_i <= FSM_CS_RAM;
--    eth_cs_i <= FSM_CS_ETH;
--    pb_oe_i <= fsm_oe_ior;
--    pb_we_i <= fsm_we_iow;
--  data signal 
--    pb_d_i <= data_fix;
--    data_out <= pb_d_o;
--    tristate_i <= FSM_tristate;
               
--   end if;
-- end process;

 a_pad: for i in 0 to 17 generate
   a_pad: FDCE port map(
     C => OPB_Clk,
     CLR => OPB_Rst,
     CE => '1',
     D => addr_s32(i),
     Q => pb_a_i(i)
  );     
 end generate a_pad;

 ub_pad : FDPE port map (
   C => OPB_Clk,
   PRE => OPB_Rst,
   CE => '1',
   D => ub,
   Q => pb_ub_i
   );
 ul_pad : FDPE port map (
   C => OPB_Clk,
   PRE => OPB_Rst,
   CE => '1',
   D => lb,
   Q => pb_lb_i
   );

 CS_RAM_pad : FDPE port map (
   C => OPB_Clk,
   PRE => OPB_Rst,
   CE => '1',
   D => FSM_CS_RAM,
   Q => ram_cs_i
   );

 CS_ETH_pad: FDPE port map (
   C => OPB_Clk,
   PRE => OPB_Rst,
   CE => '1',
   D => FSM_CS_ETH,
   Q => eth_cs_i
   );
 
 oe_pad: FDPE port map (
   C => OPB_Clk,
   PRE => OPB_Rst,
   CE => '1',
   D => fsm_oe_ior,
   Q => pb_oe_i
   );
 
 we_pad : FDPE port map (
   C => OPB_Clk,
   PRE => OPB_Rst,
   CE => '1',
   D => fsm_we_iow,
   Q => pb_we_i
   );   
 
d_pad: for i in 0 to 15 generate

di_pad : FDPE port map (
   C => OPB_Clk,
   PRE => OPB_Rst,
   CE => '1',
   D => data_fix(i),
   Q => pb_d_i(i)
   );
dt_pad :  FDPE port map (
   C => OPB_Clk,
   PRE => OPB_Rst,
   CE => '1',
   D => FSM_tristate,
   Q => tristate_i(i)
   );

do_pad : FDPE port map (
   C => OPB_Clk,
   PRE => OPB_Rst,
   CE => '1',
   D => pb_d_o(i),
   Q => data_out(i)
   );

--pb_d_i(0) <= '1';
--tristate_i(0) <= '0';
--pb_d_i(1) <= '0';
--tristate_i(1) <= '0';


end generate;
   
 
--  OPB DataPath Write

process (OPB_Rst, OPB_Clk)              -- write
begin  -- process
   if OPB_Clk'event and OPB_Clk = '1' then
    data_low <= OPB_DBus(0 to 15);
    data_high <= OPB_DBus(16 to 31);
   end if;
   if  OPB_Rst = '1' then
         data_low  <= (others => '0');
         data_high <= (others => '0');
   end if;
end process;

process (OPB_Rst, OPB_Clk)              -- read
begin  -- process
  if OPB_Clk'event and OPB_Clk = '1' then
    if sl_ld_hi = '0' and sl_ld_lo = '0' then 
      Sln_Dbus <= (others => '0') ;
    else
      if sl_ld_hi = '1' then
        Sln_Dbus(16 to 31) <= data_out;
      end if;
      if sl_ld_lo = '1' then
        Sln_Dbus(0 to 15) <= data_out;
      end if;
    end if;
  end if;

if  OPB_Rst = '1' then
  Sln_DBus <= (others => '0');
end if;  
  
end process;
  

with sel32 select
   data_fix <=
   	data_low  when '0',
	data_high when others;
  
   
   ADDR_PADS : for m in 0 to 17 generate   -- C_SRAM_AWIDTH-1 generate
    begin
      addr_pad : component OBUF_F_24 port map (
        I => pb_a_i(m),
        O => PB_A(m));
    end generate ADDR_PADS;

   
   PB_UB_PAD :OBUF_F_24 port map(
      O => PB_UB_BHE,
      I => pb_ub_i);

    PB_LB_PAD :OBUF_F_24 port map(
      O => PB_LB_AEN,
      I => pb_lb_i);

   RAM_CE_PAD :OBUF_F_24 port map(
      O => RAM_CE,
      I => ram_cs_i);

    ETH_CE_PAD :OBUF_F_24 port map(
      O => ETH_CS,
      I => eth_cs_i);

     PB_OE_PAD :OBUF_F_24 port map(
      O => PB_OE_IOR,
      I => pb_oe_i);

    PB_WE_PAD :OBUF_F_24 port map(
      O => PB_WE_IOW,
      I => pb_we_i);

    DATA_PADS : for m in 0 to 15 generate -- C_SRAM_DWIDTH-1 generate
    begin
      addr_pad : component IOBUF_F_24 port map (
        I => pb_d_i(m),
        T => tristate_i(m),
        IO => PB_D(m),
        O => pb_d_o(m));
    end generate DATA_PADS;






  

 
-- unused outputs

  Sln_errAck  <= '0';
  Sln_retry   <= '0';
  Sln_toutSup <= '0';
                                                                 -- to zero

  --changed RAM_DWIDTH TO SRAM_DWIDTH


  --in the below we changed RAM_WIDTHS TO SRAM_WIDTHS
  -- we changed it Awidth to ETH_Awidth


   
  -- Sequential part of the FSM
  fsm_seq : process(OPB_Clk, OPB_Rst)
  begin
    if OPB_Rst = '1' then
      present_state <= Idle;
    elsif OPB_Clk'event and OPB_Clk = '1' then
      present_state <= next_state;
    end if;
  end process fsm_seq;

  -- Combinational part of the FSM
    fsm_comb : process(present_state, ETHcs, RAMcs, OPB_Select, rnw, be)
  begin

    -- Default values
    FSM_tristate <= '1';
    fsm_we_iow   <= '1';
    fsm_oe_ior   <= '1';
    FSM_CS_RAM   <= '1';
    FSM_CS_ETH   <= '1';
    Sln_xferAck <= '0';
    sel32 <= '0';
    sl_ld_hi <= '0';
    sl_ld_lo <= '0';
    ub <= '1';
    lb <= '1';
             
      case present_state is

        when Idle =>
               if RAMcs = '1' then
                  next_state <= Select_Ram;
                elsif ETHcs = '1' then
                  next_state <= Select_Ethernet;
                else
                  next_state <= Idle;
                end if;
                   

         when Select_Ram =>
          if OPB_Select = '1' then
            if rnw = '1' then      --READING
              fsm_we_iow <= '1';
              FSM_CS_RAM <= '0';          
              fsm_oe_ior <= '0';
              ub <= ub_tmp;
              lb <= lb_tmp;
              if be = "1111" then
                next_state <= read_ram_32_a;
              else
                next_state <= read_ram_16_a; 
              end if;
            else                                        --WRITING
              FSM_CS_RAM <= '0';
              fsm_we_iow <= '0';
              FSM_tristate <= '0';
                ub <= ub_tmp;
              lb <= lb_tmp;
              if be = "1111"  then
                next_state <= write_ram_32;
              else
                next_state <= Xfer ;
              end if;
            end if;
            else
              next_state <= Idle;
            end if;

        when read_ram_16_a =>
               if OPB_Select = '1' then
                 FSM_CS_RAM <= '0';
                 fsm_oe_ior <= '0';
                   ub <= ub_tmp;
              lb <= lb_tmp;
                 next_state <= read_ram_16_b;
               else
                 next_state <= Idle;
               end if;

        when read_ram_16_b =>
             if OPB_Select = '1' then
                 FSM_CS_RAM <= '0';
                 fsm_oe_ior <= '0';
                 sl_ld_hi <= '1';
                 sl_ld_lo <= '1';
                   ub <= ub_tmp;
              lb <= lb_tmp;
                 next_state <= Xfer;
                 else
                   next_state <= Idle;
                 end if;

        when read_ram_32_a =>
                if OPB_Select = '1' then
                  FSM_CS_RAM <= '0';
                  fsm_oe_ior <= '0';
                  sel32 <= '1';
                    ub <= ub_tmp;
              lb <= lb_tmp;
                  next_state <= read_ram_32_b;
                else
                  next_state <= Idle;
                end if;

        when read_ram_32_b =>
                if OPB_Select = '1' then
                  FSM_CS_RAM <= '0';
                  fsm_we_iow <= '0';
                  sl_ld_lo <= '1';
                    ub <= ub_tmp;
              lb <= lb_tmp;
                  next_state <= read_ram_32_c;
                else
                  next_state <= Idle;
                end if;


        when read_ram_32_c =>
                if OPB_Select = '1' then
                  FSM_CS_RAM <= '0';
                  fsm_we_iow <= '0';
                  sl_ld_hi <= '1';
                    ub <= ub_tmp;
              lb <= lb_tmp;
                  next_state <= Xfer;
                else
                  next_state <= Idle;
                end if;       
                 
        when write_ram_32 =>
                if OPB_Select = '1' then
                  FSM_CS_RAM <= '0';
                  fsm_we_iow <= '0';
                  sel32 <= '1';
                  FSM_tristate <= '0';
                    ub <= ub_tmp;
              lb <= lb_tmp;
                  next_state <= Xfer;
                else
                  next_state <= Idle;
                end if;

-- ethenet FSM

        when Select_Ethernet =>
                if OPB_Select = '1' then
		    FSM_CS_ETH <= '0';
                    ub <= '0';
                    lb <= '0';
		 if rnw = '1' then               --READING from ETHERNET
               --    fsm_oe_ior <= '0';
                   next_state <= Read_ETH_a;
                   else                            --WRITING to the ETHERNET
                  --   FSM_tristate <= '0';
                 --    fsm_we_iow <= '0';
                     next_state <= Write_ETH_a;
                  end if;
                else
                  next_state <= Idle;
                end if;

       
        when write_ETH_a=>
                if OPB_Select = '1' then
                  FSM_CS_ETH <= '0';
                  ub <= '0';
                  lb <= '0';
                  fsm_we_iow <= '0';
                  FSM_tristate <= '0';
                  next_state <= Write_ETH_b;
                else
                  next_state <= Idle;
                end if;

        when write_ETH_b =>
                if OPB_Select = '1' then
                  FSM_CS_ETH <= '0';
                  fsm_we_iow <= '0';
                  FSM_tristate <= '0';
                  ub <= '0';
                  lb <= '0';
                  next_state <= Write_ETH_c;
                else
                  next_state <= Idle;
                end if;

         when write_ETH_c =>
                if OPB_Select = '1' then
                  FSM_CS_ETH <= '0';
                  FSM_tristate <= '0';
                  fsm_we_iow <= '0';
                  ub <= '0';
                  lb <= '0';
                  next_state <= Write_ETH_d;
                else
                  next_state <= Idle;
                end if;                     

        when write_ETH_d=>
                if OPB_Select = '1' then
                  FSM_CS_ETH <= '0';
               --   fsm_we_iow <= '0';
                  FSM_tristate <= '0';
                  ub <= '0';
                  lb <= '0';
                  next_state <= Xfer;
                else
                  next_state <= Idle;
                end if;

--         when write_ETH_e=>
  --              if OPB_Select = '1' then
   --               FSM_CS_ETH <= '0';
                --  fsm_we_iow <= '0';    -- iow on even in the last state
     --             FSM_tristate <= '0';
       --           next_state <= Xfer;
         --       else
           --       next_state <= Idle;
             --   end if;


        when Read_ETH_a=>
                if OPB_Select = '1' then
                  FSM_CS_ETH <= '0';
                  fsm_oe_ior <= '0';
                  ub <= '0';
                  lb <= '0';
                  next_state <= Read_ETH_b;
                else
                  next_state <= Idle;
                end if;

        when Read_ETH_b=>
                if OPB_Select = '1' then
                  FSM_CS_ETH <= '0';
                  fsm_oe_ior <= '0';
                  ub <= '0';
                  lb <= '0';
                  next_state <= Read_Eth_c;
                else
                  next_state <= Idle;
                end if;

        when Read_ETH_c=>
                if OPB_Select = '1' then
                  FSM_CS_ETH <= '0';
                  fsm_oe_ior <= '0';
                  ub <= '0';
                  lb <= '0';
                  next_state <= Xfer;
                else
                  next_state <= Idle;
                end if;


           when Read_ETH_d=>
                 if OPB_Select = '1' then
                   FSM_CS_ETH <= '0';
--                   fsm_oe_ior <= '0';
                   ub <= '0';
                   lb <= '0';
                   next_state <= Xfer;
                 else
                   next_state <= Idle;
                 end if;


--           when Read_ETH_e=>
--               if OPB_Select = '1' then
--                  FSM_CS_ETH <= '0';
--                   sl_ld_hi <= '1';
--                   sl_ld_lo <= '1';
--                   next_state <= Xfer;
--                  else
--                    next_state <= Idle;
--                 end if;

     -- State encoding is critical here: xfer must only be true here
        when Xfer =>            -- Actual Xfer
                Sln_xferAck <= '1';
                next_state <= Idle;
                     
      end case;
  end process fsm_comb;


  
end Behavioral;

-- Local Variables:
-- compile-command: "ghdl -a opb_bram.vhd"
-- End:
